package lemonade;
import game.Utility;

import java.util.Vector;

/**
 * A class for calculating utilities.
 * To calculate the utility of a player, there 
 * @author maz
 *
 */
public class LemonadeUtility implements Utility{
  /**
   * The size of the island: also the number of actions.
   */
  private IslandLogic logic;
  
  
  
  /**
   * Create a utility function for an island of a certain size.
   * @param islandSize the size of the island (number of actions)
   */
  public LemonadeUtility(int islandSize){
    logic = new IslandLogic(islandSize);
  }
  
  /**
   * Get the size of the island (number of actions)
   * @return the size of the island (number of actions)
   */
  public int getIslandSize(){
    return logic.getIslandSize();
  }
  
  
  
  /**
   * Get the factorial of an integer.
   * Used to keep the utilities from being fractional.
   * @param n
   * @return
   */
  private int getFactorial(int n){
    if (n<=1){
      return n;
    }
    return getFactorial(n-1)*n;
  }
  
  /**
   * An internal function for getting the utility of a single player 
   * given the players in sorted order.
   * @param positionsSorted
   * @param index
   * @return
   */
  private int getUtility(Vector<Action> positionsSorted, int index){
    int range = logic.getDistanceLeft(positionsSorted,index)+
    logic.getDistanceRight(positionsSorted,index);
    int multiplier = getFactorial(positionsSorted.size())/logic.numberOfPlayersOnTop(positionsSorted, index);
    return range * multiplier;
  }
  
  
  
  
  
  /**
   * Each player plays an action between 0 and islandSize-1 inclusive.
   * First, we determine the order of the players around the island.
   * Each player's utility is proportional to the distance from the player
   * to his left to the player to his right, divided amongst all
   * the players on top of the player.
   * 
   */
  public void getUtility(int[] positions, int[] result){
    assert(positions!=null);
    assert(result!=null);
    assert(result.length==positions.length);
    for(int player=0;player<positions.length;player++){
      assert(positions[player]>=0 && positions[player]<getIslandSize());
    }
    
    Vector<Action> positionsSorted = logic.getPositionsSorted(positions);

    
    for(int i=0;i<positionsSorted.size();i++){
      int currentPlayer = positionsSorted.get(i).player;
      result[currentPlayer]=getUtility(positionsSorted,i);
    }
  }
  
  /**
   * Each player plays an action between 0 and islandSize-1 inclusive.
   * First, we determine the order of the players around the island.
   * Each player's utility is proportional to the distance from the player
   * to his left to the player to his right, divided amongst all
   * the players on top of the player.
   * 
   */
  public int[] getUtility(int[] positions){
    int[] result = new int[positions.length];
    getUtility(positions,result);
    return result;
    
  }
  
  public boolean isSymmetric(){
    return true;
  }
  
  public Utility swapPlayers(int i,int j){
    return this;
  }
}
