package strategy;

import game.Game;

/**
 * A strategy based on (Zinkevich, 2003) (gradient descent).
 * @author maz
 *
 */
public class GIGA extends UtilityBasedStrategy{
  double[] currentStrategy;
  double learningRate;
  public GIGA(Game g, long seed, String options) {
    super(g, seed);
    currentStrategy = new double[numActions];
    projectCurrentStrategy();
    learningRate = 0.001;
  }

  @Override
  public int getAction() {
    double dart = r.nextDouble();
    for(int i=0;i<currentStrategy.length;i++){
      dart-=currentStrategy[i];
      if (dart<0.0){
        return i;
      }
    }
    return 0;
  }

  @Override
  public void observeUtilities(int[] possibleUtilities, int currentAction) {
    for(int i=0;i<currentStrategy.length;i++){
      currentStrategy[i]+=possibleUtilities[i]*learningRate;
    }
    projectCurrentStrategy();
  }

  public void projectCurrentStrategy(){
    projectVector(currentStrategy);
  }
  
  public void setVector(double[] vector){
    System.arraycopy(vector, 0, currentStrategy, 0, currentStrategy.length);
  }
  
  public double[] getVector(){
    return currentStrategy;
  }
  
  public static void projectVector(double[] currentStrategy) {
    double total=0;
    // The actions that are still being used.
    boolean[] stillLive = new boolean[currentStrategy.length];
    for(int i=0;i<currentStrategy.length;i++){
      stillLive[i]=true;
    }
    int numStillLive = currentStrategy.length;
    
    for(int i=0;i<currentStrategy.length;i++){
      total+=currentStrategy[i];
    }
    
    double necessaryChange = (total-1.0)/((double)numStillLive);
    
    while(true){
      double amountBelow = 0;
      for(int i=0;i<currentStrategy.length;i++){
        if (stillLive[i]){
          currentStrategy[i]-=necessaryChange;
          if (currentStrategy[i]<0){
            amountBelow-=currentStrategy[i];
            currentStrategy[i]=0;
            stillLive[i]=false;
            numStillLive--;
          }
        } 
      }
      if (amountBelow==0){
        break;
      }
      necessaryChange = amountBelow/((double)numStillLive);
    }
    
    
  }

}
