package strategy;

import game.Game;




public class InternalRegretMatchingStrategy extends UtilityBasedStrategy{
  /**
   * regrets[i][j] is the regret of not playing j instead of i.
   */ 
  int[][] regrets;
  public InternalRegretMatchingStrategy(Game g, long seed, String options) {
    super(g,seed);
    this.regrets = new int[numActions][numActions];
  }
  
  

  public int[][] getRegrets(){
    return regrets;
  }

  public void setRegrets(int[][] values){
    assert(regrets.length==numActions);
    for(int i=0;i<numActions;i++){
      assert(regrets[i].length==numActions);
      for(int j=0;j<numActions;j++){
        regrets[i][j]=values[i][j];
      }

    }
    
  }

  @Override
  public int getAction() {
    int totalDraw = Math.max(2*getMaxRegret(),1);
    int currentAction = r.nextInt(numActions);
    for(int i=0;i<100;i++){
      int dart = r.nextInt(totalDraw);
      for(int nextPossible=0;nextPossible<numActions;nextPossible++){
        if (regrets[currentAction][nextPossible]>0){
          dart-=regrets[currentAction][nextPossible];
          if (dart<=0){
            currentAction=nextPossible;
            break;
          }
        }
      }
    }
    return currentAction;
  }
  
  
  /**
   * Gets the maximum total positive regret of an action.
   * The total positive regret for an action is the sum of the
   * regrets of all the actions that one could have played instead
   * that would have done better.
   * @return the maximum total positive regret of an action
   */
  private int getMaxRegret() {
    int maxSoFar = 0;
    assert(regrets.length==numActions);
    for(int i=0;i<numActions;i++){
      int total=0;
      assert(regrets[i].length==numActions);
      for(int j=0;j<numActions;j++){
        if (regrets[i][j]>0){
          total+=regrets[i][j];
        }
      }
      if (total>maxSoFar){
        maxSoFar = total;
      }

    }
    return maxSoFar;
  }



  @Override
  public void observeUtilities(int[] possibleUtilities, int actionTaken) {
    for(int i=0;i<possibleUtilities.length;i++){
      regrets[actionTaken][i] += possibleUtilities[i]-possibleUtilities[actionTaken];
    }
  }
  
  
  
}
